/**
 *
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
/*
 * Version: @(#) 1.21 SDO/ws/code/sdo/src/com/ibm/websphere/sdox/SDOUtil.java, SDO.impl, SCA101X.SDO, wps1019.02 10/05/05 18:41:35 [5/10/10 22:55:32]
 *
 * IBM Confidential OCO Source Material
 * 5724-J08, 5724-I63, 5724-H88, 5724-H89, 5655-N01, 5733-W61 (C) COPYRIGHT International Business Machines Corp. 2009
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 */
package com.ibm.websphere.sdox;

import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.apache.xerces.xs.XSSimpleTypeDefinition;
import org.apache.xerces.xs.XSTypeDefinition;

import com.ibm.wsspi.sdox.XSMetaData;
import com.ibm.xml.sdo.SDOXConstants;
import com.ibm.xml.sdo.dp.SDODataProvider;
import com.ibm.xml.sdo.helper.HelperContextImpl;
import com.ibm.xml.sdo.helper.XSDHelperImpl;
import com.ibm.xml.sdo.model.ChangeSummaryCData;
import com.ibm.xml.sdo.model.XMLDocumentImpl;
import com.ibm.xml.sdo.type.SDOXProperty;
import com.ibm.xml.sdo.type.SDOXType;
import com.ibm.xml.sdo.util.SDO2XMLHelper;
import com.ibm.xml.sdo.util.XML2SDOHelper;
import com.ibm.xml.xci.CData;
import com.ibm.xml.xci.Cursor;
import com.ibm.xml.xci.serializer.SerializeParam;
import commonj.sdo.ChangeSummary;
import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;
import commonj.sdo.helper.HelperContext;
import commonj.sdo.helper.SDO;
import commonj.sdo.helper.XSDHelper;

/**
 * This class provides convenient static utility functions for implementation extended APIs
 * @ibm-api
 */
public final class SDOUtil
{
    /** 
     * URI of the DataTypeWrapper Type
     */
    final public static String DATATYPE_WRAPPER_URI = SDOXConstants.INTERNAL_SDO_NS;
    
    /**
     * SCA default HelperContext identifier
     */
    final public static String SCA_DEFAULT_SCOPE = "sca-default";
 
    /**
     * The HelperContext option OPTION_CHANGE_SUMMARY enables the change summary feature. 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "true".
     */
    final public static String OPTION_CHANGE_SUMMARY = HelperContextImpl.CHANGE_SUMMARY_OPTION;

    /**
     * The HelperContext option OPTION_SCHEMA_ERROR_HANDLER specifies the schema error handler to be used by the HelperContext instance. 
     * <br><br>Value must be an instance of {@link org.w3c.dom.DOMErrorHandler}.
     */
    final public static String OPTION_SCHEMA_ERROR_HANDLER = HelperContextImpl.SCHEMA_ERROR_HANDLER;
    
    /**
     * The HelperContext option OPTION_SCHEMA_ERROR_FAILFAST enables the schema fail fast mode. 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_SCHEMA_ERROR_FAILFAST = HelperContextImpl.SCHEMA_ERROR_FAILFAST;
    
    /**
     * The HelperContext option OPTION_SCHEMA_FULL_CHECKING enables full checking of schema constraints when loading schema documents.
     * <br><br>Value should be either "true" or "false".  <br>The default value is "true".
     */
    final public static String OPTION_SCHEMA_FULL_CHECKING = "http://www.ibm.com/xml/xci/schema/full-checking"; // Post GA from Constants.SCHEMA_FULL_CHECKING
    
    /**
     * The HelperContext option OPTION_XML_ERROR_HANDLER specifies the xml instance error handler to 
     * be used by the HelperContext instance. 
     * <br><br>Value must be an instance of {@link org.w3c.dom.DOMErrorHandler}.
     */
    final public static String OPTION_XML_ERROR_HANDLER = HelperContextImpl.XML_ERROR_HANDLER;
    
    /**
     * The HelperContext option OPTION_SAVE_USE_DEFAULT_NS specifies whether or not a QName in a namespace 
     * can be serialized in the default namespace.  
     * <br><br>Value should be either "true" or "false".  <br>The default value is "true".
     */
    final public static String OPTION_SAVE_USE_DEFAULT_NS = SerializeParam.USE_DEFAULT_NS;
    
    /**
     * The HelperContext option OPTION_SAVE_STRIP_WHITESPACE specifies whether to strip whitespace ("true") from 
     * serialized output or not ("false").  
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_SAVE_STRIP_WHITESPACE = SerializeParam.STRIP_WHITESPACE;
    
    /**
     * The HelperContext option OPTION_SAVE_REQUIRED_PREFIXES specifies a list of QName containing prefix and URI. 
     */
    final public static String OPTION_SAVE_REQUIRED_PREFIXES = SerializeParam.REQUIRED_PREFIXES;
    
    /**
     * The HelperContext option OPTION_SAVE_FORCE_NS_DECLARATIONS specifies a list of QName that would always be serialized. 
     */
    final public static String OPTION_SAVE_FORCE_NS_DECLARATIONS = SerializeParam.FORCE_NS_DECLARATIONS;
    
    /**
     * The HelperContext option OPTION_SAVE_OMIT_XML_DECL specifies whether or not to omit the xml declaration when
     * serializing instance documents. 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_SAVE_OMIT_XML_DECL = SerializeParam.OMIT_XML_DECLARATION;
    
    /**
     * The HelperContext option OPTION_SAVE_INDENT specifies whether additional whitespace may be added during serialization. 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_SAVE_INDENT = SerializeParam.INDENT;
    
    /**
     * The HelperContext option OPTION_SAVE_INDENT_LINE_SEPARATOR contains the line separator value to use when the
     * option OPTION_SAVE_INDENT == "true". 
     * <br><br>The default value is "\n".
     */
    final public static String OPTION_SAVE_INDENT_LINE_SEPARATOR = SerializeParam.INDENT_LINE_SEPARATOR;
    
    /**
     * The HelperContext option OPTION_SAVE_STANDARD_CHANGE_SUMMARY specifies whether or not the change summary information
     * is serialized using the standard XML format described in the SDO specification.
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_SAVE_STANDARD_CHANGE_SUMMARY = "STANDARD_CHANGE_SUMMARY";
    
    /**
     * The HelperContext option OPTION_SAVE_DISCARD_DEFAULT_ATTRIBUTE instructs the XML serializer to save the
     * default attribute value specified in the XSD.
     * <br><br>Value should be either boolean true or false. <br>The default value is true.
     */
    final public static String OPTION_SAVE_DEFAULT_ATTRIBUTE = "discard-default-attr"; // Post GA from SerializeParam.DISCARD_DEFAULT_ATTRIBUTE
    
    /**
     * The HelperContext option OPTION_FAST_SERIALIZATION will improve the speed of serialization operations by 
     * caching namespace information. 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "true".
     */
    final public static String OPTION_FAST_SERIALIZATION = SDOXConstants.FAST_SERIALIZATION_FEATURE;

    /**
     * The HelperContext option OPTION_EAGER_ON_LOAD will materialize the entire document/data during the 
     * "loading phase" - i.e., during the load() call.  Any well-formedness 
     * errors are reported immediately, but no validity errors are reported because 
     * the data is simply loaded, not validated.  As a side note, this happens in a document order, so there are no 
     * skipped subtrees.
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_EAGER_ON_LOAD = SDOXConstants.EAGER_ON_LOAD_FEATURE;
    
    /**
     * The HelperContext option OPTION_EAGER_BUFFER_LOAD will load the entire document to its internal buffer but not
     * necessarily parse its contents
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_EAGER_BUFFER_LOAD = SDOXConstants.EAGER_BUFFER_LOAD_FEATURE;
    
    /**
     * The HelperContext option OPTION_LOAD_SKIP_SUBTREE will materialize the root level of the document/data during the 
     * "loading phase" - i.e., during the load() call.  The remaining portions of the 
     * document/data will be loaded "as needed". 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_LOAD_SKIP_SUBTREE = SDOXConstants.LAZY_SKIP_FEATURE;

    /**
     * The HelperContext option OPTION_VALIDATE_ON_LOAD is a superset of OPTION_EAGER_ON_LOAD in that it will perform
     * a document schema validation in addition to the loading operation discussed in OPTION_EAGER_ON_LOAD.  At the
     * conclusion of the "loading phase", when this option is enabled ("true"), the document/data will have been loaded in its 
     * entirety and both well-formedness and validity errors will have been reported.
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_VALIDATE_ON_LOAD = SDOXConstants.VALID_ON_LOAD_FEATURE;
    
    /**
     * The HelperContext option OPTION_LOAD_LARGE_DOCUMENT will improve de-serialization operations for large documents (> 1MB). 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_LOAD_LARGE_DOCUMENT = SDOXConstants.LARGE_DOCUMENT_FEATURE;
    
    /**
     * The HelperContext option OPTION_COPY_DISCARD_DEFAULT_ATTRIBUTE determines whether or not schema specified default attributes
     * are propagated during a document instance copy operation. 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "false".
     */
    final public static String OPTION_COPY_DISCARD_DEFAULT_ATTRIBUTE = Cursor.COPY_DISCARD_DEFAULT_ATTRIBUTE;
    
    /**
     * The HelperContext option OPTION_COPY_ON_WRITE determines whether CopyHelper.copy() will perform 
     * copy-on-write (when possible). 
     * <br><br>Value should be either "true" or "false".  <br>The default value is "true".
     */
    final public static String OPTION_COPY_ON_WRITE = Cursor.COPY_COPY_ON_WRITE;

    final public static String OPTION_BO_COMPATIBLE = HelperContextImpl.BO_COMPATIBILITY_OPTION; //internal use
    final public static String OPTION_SCOPE_MANAGER = HelperContextImpl.SCOPE_MANAGER_OPTION; //internal use

    /**
     * Create a helper context with the specified options and a randomly generated ID
     * 
     * @param Map of options to be used when constructing the new HelperContext
     * @return new HelperContext object
     */
    public static HelperContext createHelperContext(Map<String, Object> options) {
        String uuid = null;
        
        try
        {
            uuid = UUID.randomUUID().toString();
        }
        catch (Exception e)
        {
            // UUID.randomUUID() throws NPE when there is no SecureRandom service provider
            uuid = String.valueOf(Math.random());
        }
            
        // TEMPORARY: Setting to true would switch to use DOMCursorFactory that would hit DataObjectAdaptiveProxy
        if (RuntimeHelperImpl.useDOMDataProviderHelperContext)
        {
            if (options == null)
            {
                options = new HashMap();
            }
            options.put(SDODataProvider.CURSOR_FACTORY, "com.ibm.xml.xci.adapters.dom.DOMCursorFactory");
            options.put(SDODataProvider.NO_CACHE, true);
            return SDODataProvider.createHelperContext(uuid, options);
        }
        // TEMPORARY: End
        
        HelperContext hc = null;
        try
        {
            hc = SDO.getHelperContextFactory().createHelperContext(uuid, options);
        }
        catch (NoClassDefFoundError e)
        {
            if ("com.ibm.xml.xci.CursorFactory".equals(e.getMessage()))
                throw new SDOException("SDO/X failed to start because XML Feature Pack is not present!", e);
            else
                throw e;
        }
        return hc;
    }
    
    /**
     * Create a helper context with a randomly generated ID
     *  
     * @return new HelperContext object
     */
    public static HelperContext createHelperContext()
    {
        return createHelperContext(null);
    }
        
    /**
     * Returns a Map of attributes contained by the specified DataObject
     * 
     * @param DataObject to be inspected
     * @return Map of attributes
     */
    public static Map<String, Object> getAttributes(DataObject dataObject)
    {
        Map<String, Object> attrs = new HashMap<String, Object>();
        
        List<SDOXProperty> allProperties = dataObject.getInstanceProperties();
        for (SDOXProperty property : allProperties) {
            if (!property.isElement() && dataObject.isSet(property)) {
                attrs.put(property.getName(), dataObject.get(property));
            }
        }

        return attrs;
    }
    
    /**
     * Check if a data object is compliant to its XSD.
     * 
     * @param DataObject to be validated
     * @return true if valid, otherwise false
     */
    public static boolean validate(DataObject dataObject) {
        // TODO: Is there a better way to get HelperContext from DataObject?
        HelperContextImpl hc = ((SDOXType)dataObject.getType()).getTypeHelper().getHelperContext();
        return new XMLDocumentImpl(hc, dataObject, SDOXConstants.SDO_NS, "dataObject").validate();
    }
    
    /**
     * Define a group of schemas contributing to the same target namespace.
     * Multiple target namespaces can be defined as well.
     * 
     * @param xsdHelper
     * @param A list of schema files
     * @return A list of defined SDO types
     */
    public static List /* Type */ define(XSDHelper xsdHelper, List<Source> sources) {
        return ((XSDHelperImpl)xsdHelper).define(sources);
    }
    
    /**
     * Create XSD sources for types representing WSDL components - e.g., messages, parts, etc.
     * @param xsdHelper
     * @param wsdlInputStream a .wsdl stream
     */
    public static List<Source> getWSDLSources(XSDHelper xsdHelper, InputStream wsdlInputStream, String schemaLocation) {
        return ((XSDHelperImpl)xsdHelper).getWSDLSources(wsdlInputStream, schemaLocation);
    }
    
    /**
     * Convert a value of the specified datatype to its corresponding string representation.
     * @param dataType
     * @param value
     * @return string value
     */
    public static String convertToString(Type datatype, Object value)
    {
        CData cdata = SDO2XMLHelper.wrapToCData((HelperContextImpl)datatype.getHelperContext(), value, (SDOXType)datatype, null);
        return cdata.getString(1);
    }
    
    /**
     * Convert from a string representation of the specified datatype to a value of the type.
     * @param datatype
     * @param value
     * @return object value
     */
    public static Object convertFromString(Type datatype, String value)
    {
        HelperContextImpl hc = (HelperContextImpl)datatype.getHelperContext();
        CData cdata = SDO2XMLHelper.wrapToCData(hc, value,  hc.getTypeHelperImpl().getType(new QName(SDOXConstants.SDO_NS, "String")), null);
        return XML2SDOHelper.convertToSDOValue(hc.getTypeHelperImpl(), cdata, (SDOXType)datatype, null);
    }
    
    /** 
     * Sets an instance Property (annotation) on the specified Type.
     * @param type
     * @param property
     * @param value
     */
    public static void set(Type type, Property property, Object value)
    {
        ((SDOXType)type).set(property, value);
    }
    
    /** 
     * Unsets an instance Property (annotation) on the specified Type.
     * @param type
     * @param property
     */
    public static void unset(Type type, Property property)
    {
        ((SDOXType)type).unset(property);
    }

    /** 
     * Return true if mixed content is allowed for the specified Type
     * @throw {@link AssertionError} if Type is an atomic type derivation  
     * @param type
     * @return true if mixed, otherwise false.
     */
    public static boolean isMixed(Type type)
    {
        return ((SDOXType)type).isMixed();        
    }
    
    /** 
     * Return the value of the maxLength facet on the specified Type, if available
     * @param type
     * @return value of the maxLength facet or null
     */
    public static String getMaxLength(Type type)
    {
        XSTypeDefinition xstype = XSMetaData.getXSTypeDefinition(type); 
        if (xstype instanceof XSSimpleTypeDefinition)
            return ((XSSimpleTypeDefinition)xstype).getLexicalFacetValue(XSSimpleTypeDefinition.FACET_MAXLENGTH);
        return null;
    }

    /** 
     * Returns whether or not the specified property is mapped to an XML element
     * @param property
     * @return true if the property is an XML element, otherwise false.
     */
    public static boolean isElement(Property property)
    {
        return ((SDOXProperty)property).isElement();
    }
    
    /**
     * Return the namespace-qualified name of the specified Type
     * @param type
     * @return QName of the type.
     */
    public static QName getQName(Type type)
    {
        return ((SDOXType)type).getQName();
    }

    /**
     * Return the namespace-qualified name of the specified Property
     * @param property
     * @return QName of the property.
     */
    public static QName getQName(Property property)
    {
        return ((SDOXProperty)property).getQName();
    }
    
    /**
     * Return true if the specified Type is a DataTypeWrapper
     * @param type
     * @return true if the type specified is a data wrapper, otherwise false.
     */
    public static boolean isDataTypeWrapper(Type type)
    {
        return ((SDOXType)type).isDataTypeWrapper();
    }
    
    /**
     * Creates an alias name for the specified type.
     * @param type
     * @param alias name to add to associate with type
     */
    public static void addSDOAliasName(Type type, String aliasName)
    {
        ((SDOXType)type).addSDOAliasName(aliasName);
    }

    /**
     * Suspend or resume logging for the specified ChangeSummary
     */
    public static void setLogging(ChangeSummary changeSummary, boolean logging)
    {
        ((ChangeSummaryCData)changeSummary).setLogging(logging);
    }

}
