/**
 * @(#) 1.3 SSOA/ws/code/soa.sca.bindings.jms/src.stage2/com/ibm/websphere/soa/sca/wireformat/WireFormatContext.java, WAS.soa.sca.bindings.jms, SCA101.SSOA, cf051018.03 9/30/09 13:03:50 [5/10/10 22:20:26]
 *
 * IBM Confidential OCO Source Material
 * 5724-J08, 5724-I63, 5724-H88, 5724-H89, 5655-N01, 5722-WE2 (C) COPYRIGHT International Business Machines Corp. 2009
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *         
 */

package com.ibm.websphere.soa.sca.wireformat;

import java.util.List;

/**
 * @ibm-api
 */

/**
 * The Interface WireFormatContext.
 */
public interface WireFormatContext {

	/**
	 * The Enum INVOCATION_TYPE.
	 */
	public static enum INVOCATION_TYPE { REQUEST, RESPONSE, EXCEPTION }

	/**
	 * The Enum EXCEPTION_TYPE.
	 */
	public static enum EXCEPTION_TYPE { BUSINESS_EXCEPTION, RUNTIME_EXCEPTION }

	/**
	 * Gets the invocation type of the current invocation. 
	 * 
	 * @return the invocation type
	 */
	public INVOCATION_TYPE getInvocationType();

	/**
	 * Gets the exception type.
	 * 
	 * @return the exception type or <code>null</code> if the invocation type is not <code>INVOCATION_TYPE.EXCEPTION</code>
	 */
	public EXCEPTION_TYPE getExceptionType();

	/**
	 * Gets the component.
	 * 
	 * @return the component
	 */
	public String getComponent();

	/**
	 * Gets the reference.
	 * 
	 * @return the reference
	 */
	public String getReference();

	/**
	 * Gets the service.
	 * 
	 * @return the service
	 */
	public String getService();

	/**
	 * Gets the operation names.
	 * 
	 * @return the operation names
	 */
	public List<String> getOperationNames();

	/**
	 * Gets the selected operation name.
	 * 
	 * @return the selected operation name
	 */
	public String getSelectedOperationName();

	/**
	 * Gets the target operation name.
	 * 
	 * @return the target operation name
	 */
	public String getTargetOperationName();

	/**
	 * Gets the transformed data.
	 * 
	 * @return the transformed data
	 */
	public Object getTransformedData();

	/**
	 * Sets the transformed data.
	 * 
	 * @param transformedData
	 *            the new transformed data
	 */
	public void setTransformedData(Object transformedData);

	/**
	 * Gets the custom property.
	 * 
	 * @param key
	 *            the key
	 * 
	 * @return the custom property
	 */
	public Object getCustomProperty(Object key);

	/**
	 * Sets the custom property.
	 * 
	 * @param key
	 *            the key
	 * @param value
	 *            the value
	 */
	public void setCustomProperty(Object key, Object value);

	/**
	 * Gets the wire format binding context.
	 * 
	 * @return the wire format binding context
	 */
	public WireFormatBindingContext getWireFormatBindingContext();

	/**
	 * Mark business exception.
	 */
	public void markBusinessException();

	/**
	 * Mark runtime exception.
	 */
	public void markRuntimeException();

}
